#
# Cookbook Name:: imagery
# Resource:: imagery_layer
#
# Copyright 2016, OpenStreetMap Foundation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

require "yaml"

default_action :create

property :layer, String, :name_property => true
property :site, String, :required => true
property :source, String, :required => true
property :root_layer, [TrueClass, FalseClass], :default => false
property :title, String
property :copyright, String, :default => "Copyright"
property :projection, String, :default => "EPSG:3857"
property :palette, String
property :extent, String
property :background_colour, String
property :resample, String, :default => "average"
property :imagemode, String
property :extension, String, :default => "png"
property :max_zoom, Integer, :default => 18
property :url_aliases, [String, Array], :default => []
property :revision, Integer, :default => 0
property :overlay, [TrueClass, FalseClass], :default => false
property :default_layer, [TrueClass, FalseClass], :default => false

action :create do
  file "/srv/imagery/layers/#{new_resource.site}/#{new_resource.layer}.yml" do
    owner "root"
    group "root"
    mode 0o644
    content YAML.dump(:name => new_resource.layer,
                      :title => new_resource.title || new_resource.layer,
                      :url => "//{s}.#{new_resource.site}/layer/#{new_resource.layer}/{z}/{x}/{y}.png",
                      :attribution => new_resource.copyright,
                      :default => new_resource.default_layer,
                      :maxZoom => new_resource.max_zoom,
                      :overlay => new_resource.overlay)
  end

  template "/srv/imagery/mapserver/layer-#{new_resource.layer}.map" do
    cookbook "imagery"
    source "mapserver.map.erb"
    owner "root"
    group "root"
    mode 0o644
    variables new_resource.to_hash
  end

  systemd_service "mapserv-fcgi-#{new_resource.layer}" do
    description "Map server for #{new_resource.layer} layer"
    environment "MS_MAPFILE" => "/srv/imagery/mapserver/layer-#{new_resource.layer}.map",
                "MS_MAP_PATTERN" => "^/srv/imagery/mapserver/",
                "MS_DEBUGLEVEL" => "0",
                "MS_ERRORFILE" => "stderr",
                "GDAL_CACHEMAX" => "128"
    limit_nofile 16384
    memory_high "512M"
    memory_max "2G"
    user "imagery"
    group "imagery"
    exec_start_pre "/bin/rm -f /run/mapserver-fastcgi/layer-#{new_resource.layer}.socket"
    exec_start "/usr/bin/spawn-fcgi -n -b 8192 -s /run/mapserver-fastcgi/layer-#{new_resource.layer}.socket -M 0666 -P /run/mapserver-fastcgi/layer-#{new_resource.layer}.pid -- /usr/bin/multiwatch -f 4 --signal=TERM -- /usr/lib/cgi-bin/mapserv"
    private_tmp true
    private_devices true
    private_network true
    protect_system "full"
    protect_home true
    no_new_privileges true
    restart "always"
    pid_file "/run/mapserver-fastcgi/layer-#{new_resource.layer}.pid"
  end

  service "mapserv-fcgi-#{new_resource.layer}" do
    provider Chef::Provider::Service::Systemd
    action [:enable, :start]
    supports :status => true, :restart => true, :reload => false
    subscribes :restart, "template[/srv/imagery/mapserver/layer-#{new_resource.layer}.map]"
    subscribes :restart, "systemd_service[mapserv-fcgi-#{new_resource.layer}]"
  end

  directory "/srv/imagery/nginx/#{new_resource.site}" do
    owner "root"
    group "root"
    mode 0o755
    recursive true
  end

  template "/srv/imagery/nginx/#{new_resource.site}/layer-#{new_resource.layer}.conf" do
    cookbook "imagery"
    source "nginx_imagery_layer_fragment.conf.erb"
    owner "root"
    group "root"
    mode 0o644
    variables new_resource.to_hash
  end
end

action :delete do
  file "/srv/imagery/layers/#{new_resource.site}/#{new_resource.layer}.yml" do
    action :delete
  end

  service "mapserv-fcgi-layer-#{new_resource.layer}" do
    action [:stop, :disable]
  end

  file "/srv/imagery/mapserver/layer-#{new_resource.layer}.map" do
    action :delete
  end

  systemd_service "mapserv-fcgi-#{new_resource.layer}" do
    action :delete
  end

  file "/srv/imagery/nginx/#{new_resource.site}/layer-#{new_resource.layer}.conf" do
    action :delete
  end
end

def after_created
  notifies :create, "imagery_site[#{site}]"
  notifies :reload, "service[nginx]"
end
